############################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
# Generated by setup.py
###########################################################################

import time
from pyGridWare.notification.SubscriptionManager import SubscriptionManager
from pyGridWare.lifetime.ResourceLifetime import ImmediateResourceTermination
from pyGridWare.lifetime.ResourceLifetime import ScheduledResourceTermination
from pyGridWare.generated.services.stubs.basenotification.WS_BaseNotificationService import *
from pyGridWare.generated.resource.basenotification.WS_BaseNotificationService import ManagerHome
from pyGridWare.generated.resource.basenotification.WS_BaseNotificationService import NotificationProducerContext
from pyGridWare.generated.resource.basenotification.WS_BaseNotificationService import SubscriptionManagerContext


class WS_BaseNotificationServiceWSRF(WS_BaseNotificationService):

    def GetResourceContext(ps, address):
        """get a resource context"""
        return ManagerHome.getInstance().getResourceContext(ps, address)
    GetResourceContext = staticmethod(GetResourceContext)

    def NewResourceContextNotificationProducerPort(cls, url):
        """create a new resource context"""
        ctx = NotificationProducerContext()
        ctx.setAddress(url)
        home = ctx.getResourceHome()
        key = home.create()
        home.add(ctx, key)
        ctx.setResourceID(key)
        return ctx
    NewResourceContextNotificationProducerPort = classmethod(NewResourceContextNotificationProducerPort)

    def NewResourceContextSubscriptionManagerPort(cls, url):
        """create a new resource context"""
        ctx = SubscriptionManagerContext()
        ctx.setAddress(url)
        home = ctx.getResourceHome()
        key = home.create()
        home.add(ctx, key)
        ctx.setResourceID(key)
        return ctx
    NewResourceContextSubscriptionManagerPort = classmethod(NewResourceContextSubscriptionManagerPort)

    def wsa_Notify(self, ps, address, **kw):
        #http://docs.oasis-open.org/wsn/2004/06/wsn-WS-BaseNotification-1.2-draft-01.wsdl Notify
        WS_BaseNotificationService.wsa_Notify(self, ps, address)

    def wsa_GetResourceProperty(self, ps, address, **kw):
        #http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceProperties-1.2-draft-01.wsdl GetResourcePropertyRequest
        response = WS_BaseNotificationService.wsa_GetResourceProperty(self, ps, address)
        ctx = ManagerHome.getInstance().getResourceContext(ps, address)
        namespaceURI,element = self.request
        property = ctx.properties.getResourceProperty(namespaceURI,element)
        response._any = [property]
        return response

    def wsa_Subscribe(self, ps, address, **kw):
        #http://docs.oasis-open.org/wsn/2004/06/wsn-WS-BaseNotification-1.2-draft-01.wsdl SubscribeRequest
        response = WS_BaseNotificationService.wsa_Subscribe(self, ps, address)
        ctx = self.GetResourceContext(ps, address)
        assert SubscriptionManager._isInstantiated(),"SubscriptionManager is not instantiated"
        manager = SubscriptionManager.getInstance()
        response._SubscriptionReference = manager.subscribe(self.request, ctx)
        return response

    def wsa_GetCurrentMessage(self, ps, address, **kw):
        #http://docs.oasis-open.org/wsn/2004/06/wsn-WS-BaseNotification-1.2-draft-01.wsdl GetCurrentMessageRequest
        response = WS_BaseNotificationService.wsa_GetCurrentMessage(self, ps, address)
        ctx = self.GetResourceContext(ps, address)
        assert SubscriptionManager._isInstantiated(),"SubscriptionManager is not instantiated"
        manager = SubscriptionManager.getInstance()
        response._any = manager.current(self.request, ctx)
        return response

    def wsa_GetResourceProperty(self, ps, address, **kw):
        #http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceProperties-1.2-draft-01.wsdl GetResourcePropertyRequest
        response = WS_BaseNotificationService.wsa_GetResourceProperty(self, ps, address)
        ctx = ManagerHome.getInstance().getResourceContext(ps, address)
        namespaceURI,element = self.request
        property = ctx.properties.getResourceProperty(namespaceURI,element)
        response._any = [property]
        return response

    def wsa_Destroy(self, ps, address, **kw):
        #http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceLifetime-1.2-draft-01.wsdl DestroyRequest
        response = WS_BaseNotificationService.wsa_Destroy(self, ps, address)
        ctx = WS_BaseNotificationServiceWSRF.GetResourceContext(ps, address)
        ImmediateResourceTermination.Destroy(ctx)
        return response

    def wsa_SetTerminationTime(self, ps, address, **kw):
        #http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceLifetime-1.2-draft-01.wsdl SetTerminationTimeRequest
        response = WS_BaseNotificationService.wsa_SetTerminationTime(self, ps, address)
        ctx = WS_BaseNotificationServiceWSRF.GetResourceContext(ps, address)
        ScheduledResourceTermination.SetTerminationTime(ctx, self.request._RequestedTerminationTime)
        response._NewTerminationTime = time.gmtime(ctx.getTerminationTime())
        response._CurrentTime = time.gmtime(time.time())
        return response

    def wsa_PauseSubscription(self, ps, address, **kw):
        #http://docs.oasis-open.org/wsn/2004/06/wsn-WS-BaseNotification-1.2-draft-01.wsdl PauseSubscriptionRequest
        response = WS_BaseNotificationService.wsa_PauseSubscription(self, ps, address)
        ctx = self.GetResourceContext(ps, address)
        assert SubscriptionManager._isInstantiated(),"SubscriptionManager is not instantiated"
        manager = SubscriptionManager.getInstance()
        return response

    def wsa_ResumeSubscription(self, ps, address, **kw):
        #http://docs.oasis-open.org/wsn/2004/06/wsn-WS-BaseNotification-1.2-draft-01.wsdl ResumeSubscriptionRequest
        response = WS_BaseNotificationService.wsa_ResumeSubscription(self, ps, address)
        ctx = self.GetResourceContext(ps, address)
        assert SubscriptionManager._isInstantiated(),"SubscriptionManager is not instantiated"
        manager = SubscriptionManager.getInstance()
        return response

