############################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
# Generated by setup.py
###########################################################################

from pyGridWare.lifetime.ResourceLifetime import ImmediateResourceTermination
from pyGridWare.lifetime.ResourceLifetime import ScheduledResourceTermination
from pyGridWare.notification.SubscriptionManager import SubscriptionManager
from pyGridWare.generated.services.stubs.performancecounter.CounterService import *
from pyGridWare.generated.resource.performancecounter.CounterService import ManagerHome
from pyGridWare.generated.resource.performancecounter.CounterService import CounterPortTypeContext


class CounterServiceWSRF(CounterService):

    def GetResourceContext(ps, address):
        """get a resource context"""
        return ManagerHome.getInstance().getResourceContext(ps, address)
    GetResourceContext = staticmethod(GetResourceContext)

    def NewResourceContextCounterPortTypePort(cls, url):
        """create a new resource context"""
        ctx = CounterPortTypeContext()
        ctx.setAddress(url)
        home = ctx.getResourceHome()
        key = home.create()
        home.add(ctx, key)
        ctx.setResourceID(key)
        return ctx
    NewResourceContextCounterPortTypePort = classmethod(NewResourceContextCounterPortTypePort)

    def wsa_createCounter(self, ps, address, **kw):
        #http://perf.counter.com CreateCounterRequest
        response = CounterService.wsa_createCounter(self, ps, address)
        return response

    def wsa_add(self, ps, address, **kw):
        #http://perf.counter.com AddInputMessage
        response = CounterService.wsa_add(self, ps, address)
        return response

    def wsa_Destroy(self, ps, address, **kw):
        #http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceLifetime-1.2-draft-01.wsdl DestroyRequest
        response = CounterService.wsa_Destroy(self, ps, address)
        ctx = CounterServiceWSRF.GetResourceContext(ps, address)
        ImmediateResourceTermination.Destroy(ctx)
        return response

    def wsa_SetResourceProperties(self, ps, address, **kw):
        #http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceProperties-1.2-draft-01.wsdl SetResourcePropertiesRequest
        response = CounterService.wsa_SetResourceProperties(self, ps, address)
        ctx = ManagerHome.getInstance().getResourceContext(ps, address)
        if self.request._Insert is not None:
            raise NotImplemetedError, "SetResourceProperties insert not implemented"
        elif self.request._Update is not None:
            for any in self.request._Update._any:
                nspname,pname = any.typecode.nspname,any.typecode.pname
                ctx.properties.setResourceProperty(nspname, pname, any)
        elif self.request._Delete is not None:
            raise NotImplemetedError, "SetResourceProperties delete not implemented"
        else:
            raise RuntimeError, "unknown request for SetResourceProperties."
        return response

    def wsa_GetResourceProperty(self, ps, address, **kw):
        #http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceProperties-1.2-draft-01.wsdl GetResourcePropertyRequest
        response = CounterService.wsa_GetResourceProperty(self, ps, address)
        ctx = ManagerHome.getInstance().getResourceContext(ps, address)
        namespaceURI,element = self.request
        property = ctx.properties.getResourceProperty(namespaceURI,element)
        response._any = [property]
        return response

    def wsa_Subscribe(self, ps, address, **kw):
        #http://docs.oasis-open.org/wsn/2004/06/wsn-WS-BaseNotification-1.2-draft-01.wsdl SubscribeRequest
        response = CounterService.wsa_Subscribe(self, ps, address)
        ctx = self.GetResourceContext(ps, address)
        assert SubscriptionManager._isInstantiated(),"SubscriptionManager is not instantiated"
        manager = SubscriptionManager.getInstance()
        response._SubscriptionReference = manager.subscribe(self.request, ctx)
        return response

    def wsa_GetCurrentMessage(self, ps, address, **kw):
        #http://docs.oasis-open.org/wsn/2004/06/wsn-WS-BaseNotification-1.2-draft-01.wsdl GetCurrentMessageRequest
        response = CounterService.wsa_GetCurrentMessage(self, ps, address)
        ctx = self.GetResourceContext(ps, address)
        assert SubscriptionManager._isInstantiated(),"SubscriptionManager is not instantiated"
        manager = SubscriptionManager.getInstance()
        response._any = manager.current(self.request, ctx)
        return response

