############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
import os, sys, urlparse, optparse, types
from twisted.python import log
from twisted.internet import reactor
from ConfigParser import ConfigParser, NoOptionError
from ZSI.wstools.Namespaces import WSA200403


###########################################################################
# FUNCTIONS
###########################################################################

CP = None
def GetConfigParser(f='client-config.txt'):
    '''Return the global ConfigParser instance.
    '''
    global CP
    if CP is None:
        CP = ConfigParser()
        CP.read(f)
    return CP


def GetOptParser(test):
    '''Return an OptionParser instance with a few options.
    test -- configuration file name for test.
    '''
    url = None
    scheme = 'https'
    op = optparse.OptionParser()
    cp = GetConfigParser()

    kw = {}
    scheme = 'https'
    try:
        path = cp.get('Paths', test)
        host = cp.get('Location', 'host')
        port = cp.getint('Location', 'port')

        debug = cp.get('logging', 'debug')
        if not cp.getint('security', 'ssl'):
            scheme = 'http'
    except NoOptionError, ex:
        kw['required'] = True
    else:
        kw['default'] = urlparse.urlunparse((scheme, '%s:%d'%(host,port), path, '', '', ''))

    op.add_option("-u", "--url", action="store", dest="url",
        help="location of service", **kw)
    op.add_option("-d", "--debug",
        action="store", dest="debug", default=debug, 
        help="Debug Messages to file, if 1 send to stdout, if 0 then off.")
    op.add_option("-w", "--wsAddressURI",
        action="store", dest="wsAddressURI", default=WSA200403.ADDRESS,
        help="Specify URI of ws-addressing version to use.")
    op.add_option("-c", "--cert", action="store", dest="cert",
        default='%s/%s/%s' %(os.environ.get('HOME',''), '.globus', 'usercert.pem'),
        help="file path of certificate")
    op.add_option("-k", "--key", action="store", dest="key",
        default='%s/%s/%s' %(os.environ.get('HOME',''), '.globus', 'userkey.pem'),
        help="file path of key")
    op.add_option("-y", "--secureMessage", action="store", dest="secureMessage", 
        type="int", default=0, help="use secure message to sign messages.")
    op.add_option("-z", "--secureConv", action="store", dest="secureConv", default=None,
        help="To use secure conversation, provide URL to Authorization service, and negotiate key used to sign/verify messages.")
    return op


def GetFileDescriptor(filename):
    '''Returns a file descriptor or None.
    filename -- str
    '''
    if type(filename) is types.FileType:
        return filename
    if not filename or filename == '0':
        return None
    elif filename == '1':
        return sys.stdout
    return open(filename, 'w+')


def GetPortKWArgs(options):
    '''Returns a set of key word arguments for port initialization.
    options -- Options instance.
    '''
    kw = {}
    kw['wsAddressURI'] = options.wsAddressURI
    kw['url'] = options.url
    if options.url.startswith('https') is True:
        from twisted.internet import ssl
        from pyGridWare.utility.ssl import ClientContextFactory as ContextFactory
        calist = \
            filter(\
                lambda s: s, 
                map(lambda s: s.strip(), GetConfigParser().get('security', 'ca').split(' '))
            )
        kw['contextFactory'] = \
            ContextFactory(\
                options.key, options.cert, sslmethod=ssl.SSL.SSLv3_METHOD, calist=calist,
            )

    if options.secureConv:
        from pyGridWare.security.gss.Authenticate import GssAuthenticationHandler
        from pyGridWare.security.gss.GssSignatureHandler import GssSignatureHandler
        kwc = kw.copy()
        kwc['url'] = options.secureConv
        auth = GssAuthenticationHandler()
        keyname = auth.authenticate(**kwc)
        kw['sig_handler'] = GssSignatureHandler()
        kw['sig_handler'].setSecureConversation(keyname)
    elif  options.secureMessage > 0:
        from pyGridWare.security.gss.GssSignatureHandler import GssSignatureHandler
        kw['sig_handler'] = GssSignatureHandler()
        kw['sig_handler'].setSecureMessage()

    return kw


def SetUp(options):
    '''General Script SetUp.
    options -- Options instance.
    '''
    fd = GetFileDescriptor(options.debug)
    if fd is not None:
        log.startLogging(fd)


